% Created 2013-April-16
% Written by Fu Lin
% (Y,y)-minimization step in the ADMM algorithm for the convex relaxation
% of the noise-free leader selection problem.
% Inputs: the Laplacian matrix L,
%         matrix U, vector u,
%         positive number rho,
%         initial conditions Y0 and y0,
%         stopping criterion tolerance tolY, toly.
% Outputs: solution Y, y.

function [Y,y] = Yymin(L,U,u,rho,Y0,y0,tolY,toly)

n = size(L,1);

gradproj_max_iter = 100;
Y = Y0;
y = y0;
one = ones(n,1);

tolpos = 1.e-3;

% backtracking linesearch parameters
alpha = 0.3;
beta = 0.5;
    
% check if the initial condition is valid
posYy = L.*Y + diag(one - y);

if min(eig(posYy)) < 0
    error('L.*Y + diag(one - y) should be positive definite!')
end

if min(eig(Y0)) < - tolpos
    error('The initial condition for Y should be positive semidefinite!')
end

% compute the objective function
h = trace(inv(posYy)) + (rho/2)*( norm(Y-U,'fro')^2 + norm(y-u)^2 );

for gradproj_step = 1:gradproj_max_iter
    
    % compute the gradient direction with respect to Y and y
    M = (L.*Y + diag(one - y))\eye(n);
    M = (M + M')/2;
    gradY = - (M^2 .* L) + rho*(Y - U);
    gradY = (gradY + gradY')/2;
    grady = diag(M^2) + rho*(y - u);        

    % stopping criterion 
    if min(eig(Y)) > - tolpos && min(eig(gradY)) > - tolpos ...
            && trace(Y*gradY) < tolY && norm(grady)^2 < toly
        break;
    end
        
    % form a new point 
    Ynew = Y - gradY;
    
    % project the new point to the positive semidefinite cone
    [YV,YD] = eig(Ynew);
    Ynew_proj = YV*max(YD,0)*YV';
    Ynew_proj = (Ynew_proj + Ynew_proj')/2;       
    
    % form the descent feasible direction for Y
    Ydir = Ynew_proj - Y;

    % stopping criterion 
%     if norm(Ydir,'fro')^2 + norm(grady)^2 < min(tolY,toly)
%         break;
%     end

    % choosing the stepsize
    stepsize = 1;   
    
    while 1
        
        Ytemp = Y + stepsize*Ydir;
        ytemp = y - stepsize*grady;
        posYytemp = L.*Ytemp + diag(one - ytemp);
        htemp = trace(inv(posYytemp)) + (rho/2)*(norm(Ytemp-U,'fro')^2 + norm(ytemp-u)^2);        
        
        % if posYy is positive definite, Y is positive semidefinite, and 
        % there is a sufficient decrease in the objective function,
        % then we finish the linesearch; otherwise shrink the stepsize.
        
        if min(eig(posYytemp)) > 0 && min(eig(Ytemp)) >= - tolpos && ...
                h - htemp > alpha*stepsize*(norm(Ydir,'fro')^2 + norm(grady)^2)
            break;
        end
        
        stepsize = beta*stepsize;
        
        if stepsize < 1.e-10
            min(eig(posYytemp))
            min(eig(Ytemp))
            error('Extremely small stepsize in Yy-minimization step.')
        end
    end
    
%     proj_diff = norm(Y-Ytemp,'fro')^2 + norm(y-ytemp)^2;
%     if proj_diff < tol
%         break;
%     end

    Y = Ytemp;
    y = ytemp;
    h = htemp;
        
end
