% Created 2013-June-11
% Leader selection in stochastically forced consensus networks.
% Written by Fu Lin
%
% Let L be a graph Laplacian of an undirected connected graph,
% L = D - A, 
% where D and A are the degree matrix and the adjacency matrix. 
% 
% We consider two leader selection problems in consensus networks.
% 1) Noise-corrupted leader selection problem.
% Let 
%     Lbar := L + diag(kappa)
% be a diagonally strengthened graph Laplacian, obtained by adding vector
% kappa with positive elements to the main diagonal of L. 
% 
% We are interested in selecting a subset of diagonal elements of L such
% that the trace of the inverse of the strengthened Laplacian is minimized. 
% This problem can be formulated as
%
% minimize    J(x)   = trace( inv( L + diag(kappa) * diag(x) ) )
% subject to  x(i)  \in \{0,1\}
%             sum(x) = Nl
% where the optimization variable x is a Boolean-valued vector with
% cardinality Nl.
%
% 2) Noise-free leader selection problem.
% We are interested in selecting a principal submatrix of L such that the
% trace of inverse of the submatrix is minimized over all possible
% principal submatrices with a fixed dimension. 
%
% This problem can be formulated as
%
% minimize    J(x)   =  trace( inv( Lf ) )
% subject to  x(i)  \in \{0,1\}
%             sum(x) =  Nl
%
% where the optimization variable x is a Boolean-valued vector with
% cardinality Nl. The index set of nonzero elements of x identifies the rows
% and columns of the graph Laplacian L that need to be deleted in order to
% obtain Lf.
%
% We compute lower and upper bounds on the global optimal value of these
% combinatorial problems. Lower bounds are obtained from convex relaxations
% and upper bounds are obtained using a greedy algorithm.
% 
% Syntax
% Inputs: L     -> Graph Laplacian of an undirected connected graph;
%         Nl    -> The number of leaders;
%         flag  -> The indicator with 
%                  flag == 1 for the noise-corrupted formulation, 
%                  flag == 2 for the noise-free formulation;
%         kappa -> The vector with positive elements.
%
% Outputs: Jlow -> Lower bound on the global optimal value;
%          Jup -> Upper bound on the global optimal value;
%          x      -> The Boolean-valued vector x with cardinality Nl.

function [Jlow,Jup,x] = leaders(L,Nl,kappa,flag)

switch flag
    case 1
        if min(kappa) <= 0
            error('kappa needs to be a vector with positive elements')
        else
            [Jlow,Jup,x] = noisecorrupted(L,kappa,Nl);
        end
    case 2
        [Jlow,Jup,x] = noisefree(L,Nl);
    otherwise
        error(['flag needs to be either 1 or 2 '... 
            'with flag == 1 for the noise-corrupted formulation and ' ...
            'flag == 2 for the noise-free formulation.'])
end
