% Created 2013-June-1
% Written by Fu Lin
%
% Noise-corrupted leader selection problem.
% Let L be a graph Laplacian L = D - A, 
% where D and A are the degree matrix and the adjacency matrix of an
% undirected connected graph. 
% Let 
%     Lbar := L + diag(kappa)
% be a diagonally strengthened graph Laplacian, obtained by adding vector
% kappa with positive elements to the main diagonal of L. 
% 
% We are interested in selecting a subset of diagonal elements of L such
% that the trace of the inverse of the strengthened Laplacian is minimized. 
% This problem can be formulated as
%
% minimize    J(x)   = trace( inv( L + diag(kappa) * diag(x) ) )
% subject to  x(i)  \in \{0,1\}
%             sum(x) = Nl
% where the optimization variable x is a Boolean-valued vector with
% cardinality Nl.
%
% In the context of leader-follower networks subject to stochastic
% disturbances, this problem has the interpretation of selecting a
% Nl noise-corrupted leaders such that the mean-square error of the network
% is minimized.
%
% We compute lower and upper bounds on the global optimal value of this
% combinatorial problem. A lower bound is obtained from a convex relaxation
% and an upper bound is obtained using a greedy algorithm.
% 
% Inputs: L     -> Graph Laplacian of an undirected connected graph;
%         kappa -> The vector with positive elements;
%         Nl    -> The number of noise-corrupted leaders.
%
% Outputs: Jlower -> Lower bound on the global optimal value;
%          Jupper -> Upper bound on the global optimal value;
%          x      -> The Boolean-valued vector x with cardinality Nl.

function [Jlower,Jupper,x] = noise_corrupted(L,kappa,Nl)

Da = diag(kappa);
n = size(L,2);
x = zeros(n,1);
     
% compute lower bound using convex relaxation
Jlower = cvxrelax(L,Nl,Da); 

% compute upper bound using the one-leader-at-a-time greedy algorithm
[Jgreedy,leader_idx1,invL] = greedyalg(L,Nl,Da);     %#ok<ASGLU>

% improve upper bound using a swap algorithm between leaders and
% followers
[Jupper,leader_idx,swapno] = twoswap_invL(invL,leader_idx1,Da,10*n); %#ok<NASGU>
x(leader_idx) = 1;
