% Created 2013-June-1
% Written by Fu Lin
%
% Noise-free leader selection problem.
% Let L be a graph Laplacian L = D - A, 
% where D and A are the degree matrix and the adjacency matrix of an
% undirected connected graph. 
% 
% We are interested in selecting a principal submatrix of L such that the
% trace of inverse of the submatrix is minimized over all possible
% principal submatrices with a fixed dimension. 
%
% This problem can be formulated as
%
% minimize    J(x)   =  trace( inv( Lf ) )
% subject to  x(i)  \in \{0,1\}
%             sum(x) =  Nl
%
% where the optimization variable x is a Boolean-valued vector with
% cardinality Nl. Here, Lf is the principal submatrix of L obtained by
% deleting rows and columns of L determined by nonzero elements of x.
%
% In the context of leader-follower networks subject to stochastic
% disturbances, this problem has the interpretation of selecting a
% Nl noise-free leaders such that the mean-square error of the network
% is minimized.
%
% We compute lower and upper bounds on the global optimal value of this
% combinatorial problem. A lower bound is obtained from a convex relaxation
% and an upper bound is obtained using a greedy algorithm.
% 
% Inputs: L     -> Graph Laplacian of an undirected connected graph;
%         Nl    -> The number of noise-free leaders.
%
% Outputs: Jlower -> Lower bound on the global optimal value;
%          Jupper -> Upper bound on the global optimal value;
%          x      -> The Boolean-valued vector x with cardinality Nl.

function [Jlower,Jupper,x] = noise_free(L,Nl)

N = size(L,2);
x = zeros(N,1);
     
% compute lower bound using convex relaxation
% Nf is the number of followers
Nf = N - Nl;
[Xadmm,xadmm] = noisefree_convex_relaxation(L,Nf);       
Jlower = trace(inv( (L.*Xadmm) + diag(ones(N,1) - xadmm) )) - Nl;

% one-leader-at-a-time greedy algorithm
[Jgreedy,leader_idx] = greedy_noisefree_leader(L,Nl); %#ok<ASGLU>

% Swap algorithm to improve the selection of leaders
v = zeros(N,1); 
v(leader_idx) = 1;    
follower_idx = find(1-v);

Max_swaps = length(L);
[Jupper,leader_idx,swap_step] = noisefree_twoswap_sorted(L,leader_idx,follower_idx,Max_swaps); %#ok<NASGU>

x(leader_idx) = 1;
