% Created 2013-April-16
% Written by Fu Lin
% Alternating direction method of multipliers to solve the convex
% relaxation of the noise-free leader selection problem.
% Inputs: the Laplacian matrix L and the number of followers Nf,
% Outputs: vector y and matrix Y.

function [Zpos,z] = noisefree_convex_relaxation(L,Nf)

% number of nodes
n = size(L,1);

ADMM_Max_Iter = 1000;

% quiet = 1;

% initial condition for ADMM
Y = eye(n,n);
y = (1/2)*ones(n,1);
Z = Y;
z = y;
Lambda = zeros(n,n);
lambda = zeros(n,1);

% absolute and relat    ive tolerances for the stopping criterion of ADMM
eps_abs = 1.e-3;
eps_rel = 1.e-3;

tolY = 1.e-2;
toly = 1.e-3;
rho = 1;

for ADMMstep = 1:ADMM_Max_Iter

        % ========================================================
        % (Y,y)-minimization step using gradient projection method 
        % ========================================================
        U = Z - Lambda/rho;
        u = z - lambda/rho;        
        [Y,y] = Yymin(L,U,u,rho,Y,y,tolY,toly);
        % ========================================================

        % ========================================================
        % (Z,z)-minimization step: projections onto simplex sets
        % ========================================================
        V = Y + Lambda/rho;
        v = y + lambda/rho;
        
%         [Znew,znew] = Zzmin(V,v,rho,Nf);
        znew = proj_simplex_ADMM(v,Nf);
        Znew_vec = proj_simplex_ADMM(reshape(V, numel(V), 1),Nf^2);
        Znew = reshape(Znew_vec,n,n);

        % dual residual
        resZ = norm(Z - Znew,'fro');        
        resz = norm(z - znew);        
        Z = Znew;
        z = znew;

        % primal residual
        resYZ = norm(Y - Z,'fro');
        resyz = norm(y - z);
        
        % ===========================================================        

        % ==================== update dual variables ===============
        Lambda = Lambda + rho * ( Y - Z );
        lambda = lambda + rho * ( y - z );
        % =============================================================

        % stoppin criterion for ADMM
        % evaluate the primal epsilon and the dual epsilon
        eps_pri  = sqrt(n*n + n) * eps_abs + eps_rel * max(norm(Y,'fro') + norm(y), norm(Z,'fro') + norm(z));
        eps_dual = sqrt(n*n + n) * eps_abs + eps_rel * (norm(Lambda,'fro') + norm(lambda));

        if  (resYZ + resyz < eps_pri)  &&  (rho*(resZ + resz) < eps_dual)
            break;
        end    


        disp([num2str(ADMMstep),'  ',num2str(resYZ + resyz ,'%6.2E'),'  ',num2str(rho*(resZ + resz),'%6.2E'),'  ',num2str(rho)])

    % adaptively change rho
    tau = 2;
    mu = 10;
    if (resYZ + resyz) > mu*rho*(resZ + resz)
        rho = tau*rho;
    elseif rho*(resZ + resz) < mu*(resYZ + resyz)
        rho = rho/tau;
    end

    if ADMMstep == ADMM_Max_Iter
        disp('Maximum number of ADMM steps reached!')
        disp(['The primal residual is ', num2str(resYZ + resyz,'%10.4E')]);
        disp(['The dual residual is ', num2str(rho*(resZ + resz),'%10.4E')]);
    end        

end

[VZ,DZ] = eig(Z);
Zpos = VZ*max(DZ,0)*VZ';
Zpos = (Zpos + Zpos')/2;
