% Created 2013-April-16
% Written by Fu Lin
% Projection onto the simplex set in the ADMM algorithm 
% for the convex relaxation of the noise-free leader selection problem.
% Minimize (1/2)*norm(y-a)^2
% Subject to y_i \in [0,1] and sum(y) = b.
% Inputs: vector a and scalar b.
% Outputs: solution y.
% We use ADMM for this quadratic program.

function z = proj_simplex_ADMM(a,b)

ADMM_Max_Iter = 10000;
n = length(a);
one = ones(n,1);

% initial condition for ADMM
lambda = zeros(n,1);
z = zeros(n,1);
rho = 1;

% absolute and relative tolerances for the stopping criterion of ADMM
eps_abs = 1.e-6;
eps_rel = 1.e-4;

for ADMMstep = 1:ADMM_Max_Iter
        
    u = z - lambda/rho;
    
    % x-minimization step
    nu = (1/n)*(sum(a) - b - rho*(b - sum(u)));
    x  = 1/(rho+1) * ( a + rho*u - nu*one );
    resxz = norm(x-z);
        
    % z-minimization step
    v = x + lambda/rho;
    znew = zeros(n,1);
    znew = one.*(v > 1) + v.*(0<=v & v<=1);
    resz = norm(z-znew);
    z = znew;
    
    % lambda update
    lambda = lambda + rho*(x-z);
    
    eps_pri  = sqrt(n) * eps_abs + eps_rel * max(norm(x), norm(z));
    eps_dual = sqrt(n) * eps_abs + eps_rel * norm(lambda);

    if  (resxz < eps_pri)  &&  (rho*resz < eps_dual)
        break;
    end    
    
    % adaptively change rho
    tau = 2;
    mu = 10;
    if rho*resz > mu*resxz
        rho = tau*rho;
    elseif resxz < mu*rho*resz
        rho = rho/tau;
    end

%     disp([num2str(ADMMstep),'   ',num2str(resxz,'%6.1E'),'    ',num2str(resz,'%6.1E')])
        
    if ADMMstep == ADMM_Max_Iter
        disp('Maximum number of ADMM steps reached!')
        disp(['The primal residual is ', num2str(resxz,'%10.4E')]);
        disp(['The dual residual is ', num2str(rho*resz,'%10.4E')]);
    end        
        
end

